@@ -18,6 +18,4 @@ typedef StressMajorizationSmoother UniformStressSmoother;
 
 void UniformStressSmoother_delete(UniformStressSmoother sm);
 
-UniformStressSmoother UniformStressSmoother_new(int dim, SparseMatrix A, real *x, real alpha, real M, int *flag);
-
-void uniform_stress(int dim, SparseMatrix A, real *x, int *flag);
+void uniform_stress(int dim, SparseMatrix A, double *x);
