@@ -14,8 +14,8 @@
 #include <io.h>
 #endif
 
-#include <limits.h>
 #include <stdarg.h>
+#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -25,13 +25,14 @@
 
 #include <gvc/gvplugin_render.h>
 #include <gvc/gvplugin_device.h>
+#include <cgraph/alloc.h>
 #include <cgraph/agxbuf.h>
+#include <cgraph/prisize_t.h>
+#include <cgraph/unreachable.h>
 #include <common/utils.h>
 #include <gvc/gvc.h>
 #include <gvc/gvio.h>
 
-#define GNEW(t)          malloc(sizeof(t))
-
 /* #define NEW_XDOT */
 
 typedef enum {
@@ -75,7 +76,7 @@ typedef struct {
     attrsym_t *e_l_draw;
     attrsym_t *hl_draw;
     attrsym_t *tl_draw;
-    unsigned char buf[NUMXBUFS][BUFSIZ];
+    char buf[NUMXBUFS][BUFSIZ];
     unsigned short version;
     char* version_s;
 } xdot_state_t;
@@ -83,7 +84,7 @@ static xdot_state_t* xd;
 
 static void xdot_str_xbuf (agxbuf* xb, char* pfx, const char* s)
 {
-    agxbprint (xb, "%s%zu -%s ", pfx, strlen(s), s);
+    agxbprint (xb, "%s%" PRISIZE_T " -%s ", pfx, strlen(s), s);
 }
 
 static void xdot_str (GVJ_t *job, char* pfx, const char* s)
@@ -187,7 +188,7 @@ static void xdot_fillcolor (GVJ_t *job)
 
 static void xdot_style (GVJ_t *job)
 {
-    unsigned char buf0[BUFSIZ];
+    char buf0[BUFSIZ];
     char buf [128]; /* enough to hold a double */
     agxbuf xb;
     char* p, **s;
@@ -251,10 +252,7 @@ static void xdot_style (GVJ_t *job)
  */
 static void put_escaping_backslashes(Agobj_t* n, Agsym_t *sym, const char *value)
 {
-    agxbuf buf;
-
-    /* create a temporary buffer */
-    agxbinit(&buf, 0, NULL);
+    agxbuf buf = {0};
 
     /* print the given string to the buffer, escaping as we go */
     for (; *value != '\0'; ++value) {
@@ -362,14 +360,26 @@ static void xdot_end_cluster(GVJ_t * job)
 }
 
 static unsigned short
-versionStr2Version (const char* str)
+versionStr2Version (char* str)
 {
-  unsigned long u = strtoul(str, NULL, 10);
-  if (u == 0 || u > USHRT_MAX) {
-    agerr(AGWARN, "xdot version \"%s\" too long", str);
-  }
+    char c, buf[BUFSIZ];
+    int n = 0;
+    char* s = str;
+    unsigned short us;
 
-  return (unsigned short)u;
+    while ((c = *s++)) {
+	if (isdigit(c)) {
+	    if (n < BUFSIZ-1) buf[n++] = c;
+	    else {
+		agerr(AGWARN, "xdot version \"%s\" too long", str);
+		break;
+	    }
+	}
+    }
+    buf[n] = '\0';
+    
+    us = atoi(buf);
+    return us;
 }
 
 /* 
@@ -401,7 +411,7 @@ xdot_begin_graph (graph_t *g, int s_arrows, int e_arrows, format_type id)
     unsigned short us;
     char* s;
 
-    xd = GNEW(xdot_state_t);
+    xd = gv_alloc(sizeof(xdot_state_t));
 
     if (id == FORMAT_XDOT14) {
 	xd->version = 14;
@@ -481,6 +491,8 @@ static void dot_begin_graph(GVJ_t *job)
 	    attach_attrs_and_arrows(g, &s_arrows, &e_arrows);
 	    xdot_begin_graph(g, s_arrows, e_arrows, job->render.id);
 	    break;
+	default:
+	    UNREACHABLE();
     }
 }
 
@@ -524,10 +536,10 @@ static void dot_end_graph(GVJ_t *job)
     g->clos->disc.io = &io;
     switch (job->render.id) {
 	case FORMAT_PLAIN:
-	    write_plain(job, g, (FILE*)job, FALSE);
+	    write_plain(job, g, (FILE*)job, false);
 	    break;
 	case FORMAT_PLAIN_EXT:
-	    write_plain(job, g, (FILE*)job, TRUE);
+	    write_plain(job, g, (FILE*)job, true);
 	    break;
 	case FORMAT_DOT:
 	case FORMAT_CANON:
@@ -541,6 +553,8 @@ static void dot_end_graph(GVJ_t *job)
 	    if (!(job->flags & OUTPUT_NOT_REQUIRED))
 		agwrite(g, job);
 	    break;
+	default:
+	    UNREACHABLE();
     }
     g->clos->disc.io = io_save;
 }
@@ -605,11 +619,10 @@ static void xdot_color_stop (agxbuf* xb, float v, gvcolor_t* clr)
 
 static void xdot_gradient_fillcolor (GVJ_t* job, int filled, pointf* A, int n)
 {
-    unsigned char buf0[BUFSIZ];
+    char buf0[BUFSIZ];
     agxbuf xb;
     obj_state_t* obj = job->obj;
-    float angle = obj->gradient_angle * M_PI / 180;
-    float r1,r2;
+    double angle = obj->gradient_angle * M_PI / 180;
     pointf G[2],c1,c2;
 
     if (xd->version < 14) {
@@ -626,10 +639,9 @@ static void xdot_gradient_fillcolor (GVJ_t* job, int filled, pointf* A, int n)
     }
     else {
 	get_gradient_points(A, G, n, 0, 3);
-	  //r1 is inner radius, r2 is outer radius
-	r1 = G[1].x;
-	r2 = G[1].y;
-	if (angle == 0) {
+	  // r2 is outer radius
+	double r2 = G[1].y;
+	if (obj->gradient_angle == 0) {
 	    c1.x = G[0].x;
 	    c1.y = G[0].y;
 	}
@@ -639,7 +651,7 @@ static void xdot_gradient_fillcolor (GVJ_t* job, int filled, pointf* A, int n)
 	}
 	c2.x = G[0].x;
 	c2.y = G[0].y;
-	r1 = r2/4;
+	double r1 = r2 / 4;
 	agxbputc(&xb, '(');
 	xdot_point (&xb, c1);
 	xdot_num (&xb, r1);
@@ -732,8 +744,10 @@ static void xdot_polyline(GVJ_t * job, pointf * A, int n)
     xdot_points(job, 'L', A, n);
 }
 
-void core_loadimage_xdot(GVJ_t * job, usershape_t *us, boxf b, boolean filled)
+void core_loadimage_xdot(GVJ_t * job, usershape_t *us, boxf b, bool filled)
 {
+    (void)filled;
+
     emit_state_t emit_state = job->obj->emit_state;
     char buf[BUFSIZ];
     
