@@ -98,6 +98,19 @@ public:
   SVGElement() = delete;
   explicit SVGElement(SVG::SVGElementType type);
 
+  /// Add an SVG `rect` element representing the bounding box of the edge to the
+  /// element
+  void add_bbox();
+  /// Add an SVG `rect` element representing the outline bounding box of the
+  /// edge to the element. The outline bounding box is the bounding box with
+  /// stroke width taken into account.
+  void add_outline_bbox();
+  /// Add an SVG `rect` element representing the overlap between the outline
+  /// bounding box of the element and another element. The outline bounding box
+  /// is the bounding box with penwidth taken into account.
+  void add_outline_overlap_bbox(SVG::SVGElement other, double tolerance = 0);
+  /// Add an SVG `rect` element as a child to the element
+  void add_rect(SVG::SVGRect rect, std::string color);
   /// \brief Return the value of an attribute retrieved from the element and its
   /// children
   ///
@@ -143,6 +156,10 @@ public:
   /// throw an exception unless the `throw_if_bbox_not_defined` parameter is
   /// `false`.
   SVG::SVGRect bbox(bool throw_if_bbox_not_defined = true);
+  /// Return the element of the given type with the specified index found among
+  /// the element's children. Throwns an exception if there's no such element
+  /// with the specified index.
+  SVG::SVGElement &find_child(SVG::SVGElementType type, std::size_t index = 0);
   bool is_closed_shape_element() const;
   bool is_shape_element() const;
   /// Return the outline bounding box of the element. The outline bounding box
