@@ -182,6 +182,10 @@ static bool check_analyzed_svg(SVGAnalyzer &svg_analyzer,
   const auto rankdir = graph_options.rankdir;
   const auto node_shape = graph_options.node_shape;
   const auto dir = graph_options.dir;
+  const auto primitive_arrowhead_shape =
+      graph_options.primitive_arrowhead_shape;
+  const auto primitive_arrowtail_shape =
+      graph_options.primitive_arrowtail_shape;
 
   REQUIRE(svg_analyzer.graphs().size() == 1);
   auto &recreated_graph = svg_analyzer.graphs().back();
@@ -265,9 +269,10 @@ static bool check_analyzed_svg(SVGAnalyzer &svg_analyzer,
   const auto edge_stem_bbox = edge_stem.outline_bbox();
 
   // check overlap of edge stem and arrowhead
-  if (dir == "forward" || dir == "both") {
-    auto edge_arrowhead = dir == "forward" ? edge.arrow(0) : edge.arrow(1);
-    const auto edge_arrowhead_bbox = edge_arrowhead.outline_bbox();
+  if ((dir == "forward" || dir == "both") &&
+      primitive_arrowhead_shape != "none") {
+    const auto edge_arrowhead_bbox =
+        edge.arrowhead_outline_bbox(dir, primitive_arrowhead_shape);
     const auto overlap_bbox = edge_stem_bbox.intersection(edge_arrowhead_bbox);
     INFO("Edge stem and arrowhead overlap:");
     INFO(fmt::format("  width:  {:.3f}", overlap_bbox.width));
@@ -296,9 +301,9 @@ static bool check_analyzed_svg(SVGAnalyzer &svg_analyzer,
   }
 
   // check overlap of edge stem and arrowtail
-  if (dir == "back" || dir == "both") {
-    auto edge_arrowtail = edge.arrow(0);
-    const auto edge_arrowtail_bbox = edge_arrowtail.outline_bbox();
+  if ((dir == "back" || dir == "both") && primitive_arrowtail_shape != "none") {
+    const auto edge_arrowtail_bbox =
+        edge.arrowtail_outline_bbox(dir, primitive_arrowtail_shape);
     const auto overlap_bbox = edge_stem_bbox.intersection(edge_arrowtail_bbox);
     INFO("Edge stem and arrowtail overlap:");
     INFO(fmt::format("  width:  {:.3f}", overlap_bbox.width));
@@ -442,10 +447,12 @@ void test_edge_node_overlap(const graph_options &graph_options,
            // FIXME: adjust this when `curve` is fixed for penwidth
            graph_options.edge_penwidth / 2 + graphviz_bezier_clip_margin},
           {"diamond",
-           // FIXME: adjust this when `diamond` is fixed for penwidth
-           graph_options.edge_penwidth / 2 + graphviz_bezier_clip_margin},
+           [&]() { // FIXME: calculate this accurately for diamond arrow
+             const auto tip_scale = 1.8027756377319939; // empirical value
+             return graph_options.edge_penwidth / 2 * tip_scale +
+                    graphviz_bezier_clip_margin;
+           }()},
           {"dot",
-           // FIXME: adjust this when `dot` is fixed for penwidth
            graph_options.edge_penwidth / 2 + graphviz_bezier_clip_margin},
           {"icurve",
            // FIXME: adjust this when `icurve` is fixed for penwidth
@@ -456,13 +463,9 @@ void test_edge_node_overlap(const graph_options &graph_options,
              return graph_options.edge_penwidth / 2 * tip_scale +
                     graphviz_bezier_clip_margin;
            }()},
-          {"none",
-           // FIXME: adjust this when `none` is fixed for penwidth
-           graph_options.edge_penwidth / 2 + graphviz_bezier_clip_margin},
           {"normal",
            graph_options.edge_penwidth / 2 + graphviz_bezier_clip_margin},
           {"tee",
-           // FIXME: adjust this when `tee` is fixed for penwidth
            graph_options.edge_penwidth / 2 + graphviz_bezier_clip_margin},
           {"vee",
            // FIXME: adjust this when `vee` is fixed for penwidth
